@@ -35,7 +35,6 @@ class Parser(object):
                                           disjunct_cost=disjunct_cost,
                                           max_sentence_length=max_sentence_length)
         self.dictionary = Dictionary("en")
-    
     def parse_sent(self, txt):
         sent = clg.sentence_create(txt,self.dictionary._dict)
         n = clg.sentence_parse(sent, self.parse_options._po)
@@ -43,6 +42,7 @@ class Parser(object):
             linkages = [Linkage.create_from_sentence(i,sent, self.parse_options._po) for i in range(n)]
         else:
             linkages = None
+        clg.sentence_delete(sent)
         #linkgrammar will start to poop its pants if its 'resources' are exhausted.
         #This is a little ugly to check each time but it seems to be a cheap operation.
         #If the resources are exhausted, then reset them.
@@ -322,17 +322,14 @@ class Linkage(object):
     @classmethod
     def create_from_sentence(cls, idx, sentence, parse_options):
         linkage_swig_obj = clg.linkage_create(idx,sentence, parse_options)
-        linkage_obj = Linkage(linkage_swig_obj)
+        linkage_obj = Linkage(linkage_swig_obj, null_count=clg.sentence_null_count(sentence))
         clg.linkage_delete(linkage_swig_obj)
         return linkage_obj
 
-    def __init__(self, linkage_swig_obj, calculate_sub_linkages=True):
-        self.diagram = clg.linkage_print_diagram(linkage_swig_obj)
+    def __init__(self, linkage_swig_obj, calculate_sub_linkages=True, null_count=0):
         self.num_of_sublinkages = clg.linkage_get_num_sublinkages(linkage_swig_obj)
         self.num_of_words = clg.linkage_get_num_words(linkage_swig_obj)
         self.num_of_links = clg.linkage_get_num_links(linkage_swig_obj)
-        self.links_and_domains = clg.linkage_print_links_and_domains(linkage_swig_obj)
-        self.senses = clg.linkage_print_senses(linkage_swig_obj)
         self.link_cost = clg.linkage_link_cost(linkage_swig_obj)
         
         self.words = [clg.linkage_get_word(linkage_swig_obj,i) for i in range(self.num_of_words)]
@@ -340,13 +337,20 @@ class Linkage(object):
                            clg.linkage_get_link_llabel(linkage_swig_obj,i),
                            clg.linkage_get_link_rlabel(linkage_swig_obj,i),
                            self.words[clg.linkage_get_link_rword(linkage_swig_obj,i)]) for i in range(self.num_of_links)]
-
+        self.null_count = null_count
         self.link_distances = [clg.linkage_get_link_length(linkage_swig_obj, i) for i in range(self.num_of_links)]
-        self.postscript_snippet = clg.linkage_print_postscript(linkage_swig_obj, 0)
-        self.postscript = clg.linkage_print_postscript(linkage_swig_obj, 1)
         self.constituent_phrases_flat = self._get_constituent_phrases(linkage_swig_obj, flat=True)
         self.constituent_phrases_nested = self._get_constituent_phrases(linkage_swig_obj, flat=False)
 
+        """
+        these cause memory leaks. sad face
+        self.senses = clg.linkage_print_senses(linkage_swig_obj)
+        self.links_and_domains = clg.linkage_print_links_and_domains(linkage_swig_obj)
+        self.diagram = clg.linkage_print_diagram(linkage_swig_obj)
+        self.postscript_snippet = clg.linkage_print_postscript(linkage_swig_obj, 0)
+        self.postscript = clg.linkage_print_postscript(linkage_swig_obj, 1)
+        """
+
         clg.linkage_compute_union(linkage_swig_obj)
         self.sublinkages = []
         if calculate_sub_linkages:
@@ -358,6 +362,7 @@ class Linkage(object):
     def _get_constituent_phrases(self, linkage_swig_obj, flat=False):
         node = clg.linkage_constituent_tree(linkage_swig_obj)
         result = self._build_constituent_phrases(node, flat)
+        clg.linkage_free_constituent_tree(node)
         return result
 
     def _build_constituent_phrases(self, node, flat):
